==> api_server_controller-0.1.9a8-py3-none-any.whl: asc/core/tools/marketplace/control.py <==
from typing import List
from datetime import date
from datetime import timedelta
from sqlalchemy import Sequence
from sqlalchemy.exc import IntegrityError

from asc import message
from asc.core import env
from asc.core import log
from asc.models.marketplace import Tax
from asc.models.marketplace import Client
from asc.models.marketplace import Payment
from asc.core.ABC.Control import ControlTool
from asc.core.tools.mailer.control \
    import ControlToolHandler as CTHMailer
from asc.core.tools.storage.control \
    import ControlToolHandler as CTHStorage
from asc.core.tools.marketplace import data
from asc.core.tools.marketplace.mappings import PaymentMethod
from asc.core.tools.marketplace.mappings import PaymentGateway
from asc.core.tools.marketplace.mercadopago.control \
    import ControlToolHandler as CTHMercadopago
from asc.core.tools.marketplace.pagarme.data import Customer
from asc.core.tools.marketplace.pagarme.control \
    import ControlToolHandler as CTHPagarme
from asc.exceptions.marketplace import UserDoesNotExists
from asc.exceptions.marketplace import PaymentGatewayNotDefined

class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()
        self.reference = None
        self.payment_info = message.M_WITHOUT_PAYMENT_INFO
        self.payment_gateway = None

    def __check_payment_gateway(self):
        if not self.payment_gateway:
            raise PaymentGatewayNotDefined()

    def set_payment_gateway(self, gateway:PaymentGateway):
        """ Define gateway de pagamento a ser utilizado pelo controlador.
        :param gateway:
        :return:
        """
        self.payment_gateway = gateway.value

    def get_reference(self):
        """ Gera referencia para ser utilizada publicamente. """
        return Sequence('marketplace_reference_seq').next_value

    def get_payment_by_id(self, id_payment:int):
        """ Obtém pagamento da API com o gateway definido.
        :param id_payment: id do pagamento.
        :return:
        """
        self.__check_payment_gateway()

        if self.payment_gateway == PaymentGateway.mercadopago.value:
            mpg_control = CTHMercadopago()
            return mpg_control.get_payment(id_payment)

    def find_client_by_email(self, email:str) -> Client:
        """ Encontra cliente marketplace por email.
        :param email: Quantos dias para expirar.
        """
        return self.session \
            .query(Client) \
            .where(Client.email == email) \
            .first()

    def calc_card_installments(self, amount:float, id:int=None) \
            -> List[data.Installment]:
        """ Calcula parcelas para cobranças de cartão de crédito.
        :param amount: valor autorizado à cobrança.
        :param id: id do split.
        :return:
        """
        self.__check_payment_gateway()

        installments = []
        query = self.session.query(Tax) \
            .where(
            Tax.is_active.is_(True),
            Tax.id_marketplace_payment_method==PaymentMethod.card.value,
            Tax.id_marketplace_payment_gateway==self.payment_gateway) \
            .first()

        i = 1
        for row_cust_split_amount in query.cust_split_amount:
            installments.append(data.Installment(
                count=i,
                total=amount,
                tax=[
                    (query.cust_process_type, query.cust_process_amount),
                    (query.cust_split_type, row_cust_split_amount)
                ],
                subtract=False,
            ))

            i += 1

        return installments

    def create_transaction_pix(self, client:Client, amount:int):
        """ Cria uma transação PIX com o gateway definido.
        :param client: cliente pagador da transação.
        :param amount: valor da transação, inteira em centavos.
        :return: dict contendo dados do pagamento.
        """
        self.__check_payment_gateway()

        if not client:
            raise UserDoesNotExists(message.E_NOT_FOUND_CLIENT)

        if self.payment_gateway == PaymentGateway.pagarme.value:
            pagarme = CTHPagarme()
            payment = pagarme.create_transaction_pix(
                customer=Customer(
                    name=client.name,
                    email=client.email,
                    phone_numbers=[client.phone],
                    documents=[
                        {'type': 'cpf', 'number': client.document}
                    ],
                    external_id=str(client.id_marketplace_client)
                ),
                amount=amount,
                info=self.payment_info
            )

        storage = CTHStorage()
        storage.set_fold('payments')

        payment['qr_link'] = storage.save_qrcode(
            payment.get('qr'), payment.get('id')
        )

        return payment

    def send_transaction_pix_email(self, client:Client, payment:dict):
        """
        Envia email para cliente, solicitando pagamento.
        :param client: dados do cliente.
        :param payment: dados do pagamento.
        :return:
        """
        payment_value = str(
            payment.get('amount') / 100)\
            .replace('.', ',')

        mailer = CTHMailer()
        mailer.send_mail(
            to=client.email,
            slug='payment-pix',
            subject=f'{message.M_SUBJECT_PAYMENT_PIX} {self.payment_info}',
            html_vars={
                'client_name': client.name,
                'payment_value': payment_value,
                'payment_qr_code': payment.get('qr'),
                'payment_qr_link': payment.get('qr_link'),
                'payment_expires': payment.get('expires'),
                'payment_description': self.payment_info,
            },
            id_client=client.id_marketplace_client
        )

        return True

==> api_server_controller-0.2.2a3-py3-none-any.whl: asc/core/tools/marketplace/control.py <==
from typing import List
from decimal import Decimal
from datetime import date
from datetime import timedelta
from sqlalchemy import Sequence
from sqlalchemy.exc import IntegrityError

from asc import message
from asc.core import env
from asc.core import log
from asc.models.marketplace import Tax
from asc.models.marketplace import Client
from asc.models.marketplace import Payment
from asc.core.ABC.Control import ControlTool
from asc.core.tools.mailer.control \
    import ControlToolHandler as CTHMailer
from asc.core.tools.storage.control \
    import ControlToolHandler as CTHStorage
from asc.core.tools.marketplace import data
from asc.core.tools.marketplace.mappings import PaymentMethod
from asc.core.tools.marketplace.mappings import PaymentGateway
from asc.core.tools.marketplace.mercadopago.control \
    import ControlToolHandler as CTHMercadopago
from asc.core.tools.marketplace.pagarme.data import Customer
from asc.core.tools.marketplace.pagarme.control \
    import ControlToolHandler as CTHPagarme
from asc.exceptions.marketplace import UserDoesNotExists
from asc.exceptions.marketplace import PaymentGatewayNotDefined


class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()
        self.reference = None
        self.payment_info = message.M_WITHOUT_PAYMENT_INFO
        self.payment_gateway: PaymentGateway = None

    def __check_payment_gateway(self):
        if not self.payment_gateway:
            raise PaymentGatewayNotDefined()

    def __sanitize_payment_value(self, amount):
        if type(amount) is int:
            amount = str(amount / 100)
        if type(amount) in [float, Decimal]:
            amount = str(amount)

        amount = amount.replace('.', ',')
        return amount

    def set_payment_info(self, payment_info: str):
        """ Preenche descrição do pagamento.
        """
        self.payment_info = payment_info

    def set_payment_gateway(self, gateway: PaymentGateway):
        """ Define gateway de pagamento a ser utilizado pelo controlador.
        :param gateway:
        :return:
        """
        self.payment_gateway = gateway

    def get_reference(self):
        """ Gera referencia para ser utilizada publicamente. """
        return Sequence('marketplace_reference_seq').next_value

    def get_payment_by_id(self, id_payment: int):
        """ Obtém pagamento da API com o gateway definido.
        :param id_payment: id do pagamento.
        :return:
        """
        self.__check_payment_gateway()

        if self.payment_gateway == PaymentGateway.mercadopago:
            mpg_control = CTHMercadopago()
            return mpg_control.get_payment(id_payment)

    def find_client_by_email(self, email: str) -> Client:
        """ Encontra cliente marketplace por email.
        :param email: Quantos dias para expirar.
        """
        return self.session \
            .query(Client) \
            .where(Client.email == email) \
            .first()

    def calc_card_installments(self, amount: float, id: int = None,
                               to_dict=False) \
            -> List[data.Installment]:
        """ Calcula parcelas para cobranças de cartão de crédito.
        :param amount: valor autorizado à cobrança.
        :param id: id do split.
        :param to_dict: retorna valores formatados como dict.
        :return:
        """
        self.__check_payment_gateway()

        installments = []
        query = self.session.query(Tax) \
            .where(
            Tax.is_active.is_(True),
            Tax.id_marketplace_payment_method == PaymentMethod.card.value,
            Tax.id_marketplace_payment_gateway == self.payment_gateway.value) \
            .first()

        i = 1
        for row_cust_split_amount in query.cust_split_amount:
            installment = data.Installment(
                count=i,
                total=amount,
                tax=[
                    (query.cust_process_type, query.cust_process_amount),
                    (query.cust_split_type, row_cust_split_amount)
                ],
                subtract=False,
            )

            if to_dict:
                installment = installment.asdict()
                installment['price'] = int(installment['price'])
                del installment['tax']

            installments.append()

            i += 1

        return installments

    def create_transaction_pix(self, client: Client, amount):
        """ Cria uma transação PIX com o gateway definido.
        :param client: cliente pagador da transação.
        :param amount: valor da transação, inteira em centavos.
        :return: dict contendo dados do pagamento.
        """
        self.__check_payment_gateway()

        if not client:
            raise UserDoesNotExists(message.E_NOT_FOUND_CLIENT)

        if self.payment_gateway == PaymentGateway.pagarme:
            pagarme = CTHPagarme()
            payment = pagarme.create_transaction_pix(
                customer=Customer(
                    name=client.name,
                    email=client.email,
                    phone_numbers=[client.phone],
                    documents=[
                        {'type': 'cpf', 'number': client.document}
                    ],
                    external_id=str(client.id_marketplace_client)
                ),
                amount=self.payment_gateway.get_amount(amount),
                info=self.payment_info
            )

        storage = CTHStorage()
        storage.set_fold('payments')
        storage_data = storage.save_qrcode(
            payment.get('qr'), payment.get('id')
        )

        payment['db_payment'].id_storage = storage_data[0]
        payment['qr_link'] = storage_data[1]
        payment['expires'] = payment['expires'].strftime('%d/%m/%Y')

        return payment

    def send_transaction_pix_email(
            self,
            client: Client,
            payment: dict,
            slug: str = 'created'):
        """
        Envia email para cliente, solicitando pagamento.
        :param client: dados do cliente.
        :param payment: dados do pagamento.
        :return:
        """
        payment_value = self.__sanitize_payment_value(
            payment.get('amount')
        )

        mailer = CTHMailer()
        mailer.send_mail(
            to=client.email,
            slug=f'{env.get("marketplace__soft_descriptor")}'
                 f'-payment-pix-{slug}',
            subject=f'{message.M_SUBJECT_PAYMENT_PIX} {self.payment_info}',
            html_vars={
                'client_name': client.name,
                'payment_value': payment_value,
                'payment_qr_code': payment.get('qr'),
                'payment_qr_link': payment.get('qr_link'),
                'payment_expires': payment.get('expires'),
                'payment_description': self.payment_info,
            },
            id_client=client.id_marketplace_client
        )

        return True
